#!/usr/bin/python3

# A little script to test all the formats we use and display the page counts.

# Bob van der Poel, Oct/2016.
# Use it free for whatever you want.
# Changed to python3, June/2020

import sys
import os
import subprocess
import atexit
import getopt

def usage():
    print("Mup-best: Calculate best pagescale option for music page.")
    sys.exit(1)

def parseOptions():
    """ Parse the command line. Permitted options are:
       filename
       -c  -- concert
       -b  -- bflat
       -e  -- eflat
       -T  -- TAB
       -F  -- Fhorn
       -B  -- BASS
       -g  -- guitar bass
       -G  -- guitar grids
       -U  -- Ukulele grids
       -nX  -- concert key, specify # of pages
    """

    keys = {'-c': ('C', '', 1),
            '-b': ('Bb', '-DBFLAT', 1),
            '-e': ('Eb', '-DEFLAT', 1), 
            '-B': ('Bass', '-DBASS', 1),
            '-g': ('Guitar Bass', '-DGBASS', 2),
            '-F': ("FHorn",  '-DFHORN',  1),
            '-G':  ("Guitar", '-DGUITAR', 1),
            '-U': ("Ukulele", '-DUKE', 1),
            '-T': ('Tab', '-DTAB', 2)}

    try:
        opts, args = getopt.gnu_getopt(sys.argv[1:], "cbeGUTFBgn:", [])
    except getopt.GetoptError:
        usage()

    keyList = []

    for o, a in opts:
        if o == '-n':
            try:
                a = int(a)
                keyList.append( ('C %s pages' % a, '', a ))
            except:
                print("Argument for -n must be an integer")
                sys.exit(1)

        elif o in keys:
            keyList.append(keys[o])

        else:
            usage()

    if not keyList:
        keyList = [ x for x in keys.values()]

    if len(args) < 1:
        print("No filename given.")
        usage()

    if len(args) > 1:
        print("Too many filenames given.")
        usage()

    return (args[0], sorted(keyList))

def cleanup():
    """ Delete any temporary work files."""

    try:
        os.unlink(inmup)
    except:
        pass


def testScale(opts, neededCount):
    """ Test various scale options.

        This does what one might do by hand in changing the staffscale value,
        doing a test compile, looking at the number of pages produced and
        looping for more.
    """

    dec       = .5  # decrement step counter (force this to be FLOAT!)
    testpoint = 5    # initial staffscale to test
    
    while 1:
        # run our test file though MUP. The test file has a line(s) in it:
        #     "staffscale=SCALE"
        
        sts, output = subprocess.getstatusoutput( "mup  %s %s -DSCALE=%1.2f " % \
                                                ( inmup, opts, testpoint) )
        if sts:
            # most likely the scale is way too big
            if testpoint>.4 and "Fatal user error" in output:
                testpoint -= dec
                continue
            else:
                print("MUP error!!!!")
                print(output)
                sys.exit(1)

        # MUP's output, a PS file, was saved in 'output'.
        # The last line of this file is always "%%Pages: XX"
        # If this changes in the future we would need to do more 
        # scanning of the PS file, but for now we just grab the
        # last word, which is the page count.

        a = int(output.rsplit()[-1])

        # I think this logic could be made much more efficient by
        # doing some kind of binary split/test. Please let me know
        # if you improve!!!!

        if a > neededCount:
            if dec > 0.04:
                testpoint += dec
                dec -= dec * .5
            testpoint -= dec
        else:
            return testpoint



###########################################

## Main line / entry point 

## Check args.

infile, keyList = parseOptions()

inmup   = "/tmp/MupInput.%s.mup" % os.getpid()

atexit.register(cleanup)

# create a test input file with all "staffscale = " replaced with "staffscale=SCALE"

if not os.path.exists(infile) and not infile.endswith('.mup'):
    infile += '.mup'
try:
    ifile = open(infile, 'r')
except IOError:
    print("Can't access file '%s'." % infile)
    sys.exit(1)

ofile = open(inmup, 'w')
mupdata = ifile.readlines()  # entire source is in sys.exit(0)

for l  in mupdata:
    if "staffscale" in l:
        l = l.rsplit('=', 1)
        l[-1] = " SCALE\n"
        l = '='.join(l)
    ofile.write(l)
ofile.close()


# The keyList is either all our known keys or a subset.
# Loop though whatever is needed.
# each tuple is: English key name
#                Mup option for generating
#                Desired page count

for key, opts, needPC in keyList:
    scale = testScale(opts, needPC)
    
    print("Best scale for %s: %1.2f" % (key, scale))


